/*
 * Copyright (c) 2007, Jim C. Nasby (decibel@decibel.org)
 * All rights reserved.
 *
 * $Id: functions.sql 31 2007-11-13 09:25:05Z decibel $
 */

CREATE OR REPLACE FUNCTION idle_stats() RETURNS SETOF record LANGUAGE plpgsql SECURITY DEFINER AS $idle_stats$
DECLARE
    v_record record;
BEGIN
    -- If there's any records with <command string not enabeld> return nothing, since we don't
    -- actually know how many idle connections there are
    IF NOT EXISTS( SELECT 1 FROM pg_stat_activity WHERE current_query = '<command string not enabled>' ) THEN
        FOR v_record IN SELECT datid
                                , sum(CASE WHEN current_query='<IDLE>' THEN 1 ELSE 0 END)::int AS idle
                                , sum(CASE WHEN current_query='<IDLE> in transaction' THEN 1 ELSE 0 END)::int AS idle_in_transaction
                            FROM pg_stat_activity
                            GROUP BY datid
        LOOP
            RETURN NEXT v_record;
        END LOOP;
    END IF;
    RETURN;
END;
$idle_stats$;

COMMENT ON FUNCTION idle_stats() IS $$Returns stats on how many connections are idle or idle in transaction. Returns nothing if stats_command_string is disabled.$$;

--		vi: expandtab sw=4 ts=4
