/*
 * Copyright (c) 2007, Jim C. Nasby (decibel@decibel.org)
 * All rights reserved.
 *
 * $Id: rrs.sql 35 2007-12-29 16:16:14Z decibel $
 */

/*
 * Useful vi commands:
 *
 * Takes all fields ending in '_min' and creates avg and max versions
 * s/\(\s*, \S*\)_min\(.*\)/&^M\1_avg\2^M\1_max\2/
 *
 * Run (min|avg|max) function on fields selected, based on tailing _ value, striping tail.
 * IE: idle_min becomes min(idle)
 * s/\v(\w*)_(min|avg|max)/\2(\1)/
 *
 * Same as above, but doesn't strip tail
 * s/\v(\w*)_(min|avg|max)/\2(\1_\2)/
 */

CREATE SCHEMA rrs_data;

CREATE TABLE rrs_data.database(
	bucket_id					int		NOT NULL REFERENCES rrs.bucket(bucket_id) ON DELETE CASCADE
	, database_id				int		NOT NULL REFERENCES raw.databases(database_id) ON DELETE CASCADE
	, num_backends_min			real
	, num_backends_avg			real
	, num_backends_max			real
	, idle_min					real
	, idle_avg					real
	, idle_max					real
	, idle_in_transaction_min	real
	, idle_in_transaction_avg	real
	, idle_in_transaction_max	real
	, commits_sec_min			real
	, commits_sec_avg			real
	, commits_sec_max			real
	, rollbacks_sec_min			real
	, rollbacks_sec_avg			real
	, rollbacks_sec_max			real
	, blocks_read_sec_min		real
	, blocks_read_sec_avg		real
	, blocks_read_sec_max		real
	, blocks_hit_sec_min		real
	, blocks_hit_sec_avg		real
	, blocks_hit_sec_max		real
	, PRIMARY KEY (bucket_id, database_id)
);
INSERT INTO rrs.source( source_name, insert_table, source_table, source_timestamptz_field, group_clause
		, insert_aggregate_fields
		, primary_aggregate
		, rrs_aggregate
	) VALUES( 'Databases', 'rrs_data.database', 'delta.database_sec', 'captured_at', 'database_id'
		, $$
		num_backends_min
		, num_backends_avg
		, num_backends_max
		, idle_min
		, idle_avg
		, idle_max
		, idle_in_transaction_min
		, idle_in_transaction_avg
		, idle_in_transaction_max
		, commits_sec_min
		, commits_sec_avg
		, commits_sec_max
		, rollbacks_sec_min
		, rollbacks_sec_avg
		, rollbacks_sec_max
		, blocks_read_sec_min
		, blocks_read_sec_avg
		, blocks_read_sec_max
		, blocks_hit_sec_min
		, blocks_hit_sec_avg
		, blocks_hit_sec_max
		$$
		-- primary_aggregate
		, $$
		min(num_backends)
		, avg(num_backends)
		, max(num_backends)
		, min(idle)
		, avg(idle)
		, max(idle)
		, min(idle_in_transaction)
		, avg(idle_in_transaction)
		, max(idle_in_transaction)
		, min(commits_sec)
		, avg(commits_sec)
		, max(commits_sec)
		, min(rollbacks_sec)
		, avg(rollbacks_sec)
		, max(rollbacks_sec)
		, min(blocks_read_sec)
		, avg(blocks_read_sec)
		, max(blocks_read_sec)
		, min(blocks_hit_sec)
		, avg(blocks_hit_sec)
		, max(blocks_hit_sec)
		$$
		-- primary_aggregate
		, $$
		min(num_backends_min)
		, avg(num_backends_avg)
		, max(num_backends_max)
		, min(idle_min)
		, avg(idle_avg)
		, max(idle_max)
		, min(idle_in_transaction_min)
		, avg(idle_in_transaction_avg)
		, max(idle_in_transaction_max)
		, min(commits_sec_min)
		, avg(commits_sec_avg)
		, max(commits_sec_max)
		, min(rollbacks_sec_min)
		, avg(rollbacks_sec_avg)
		, max(rollbacks_sec_max)
		, min(blocks_read_sec_min)
		, avg(blocks_read_sec_avg)
		, max(blocks_read_sec_max)
		, min(blocks_hit_sec_min)
		, avg(blocks_hit_sec_avg)
		, max(blocks_hit_sec_max)
		$$
	)
;

CREATE TABLE rrs_data.table(
	bucket_id							int		NOT NULL REFERENCES rrs.bucket(bucket_id) ON DELETE CASCADE
	, table_id							int		NOT NULL REFERENCES raw.tables(table_id) ON DELETE CASCADE
	, seq_scans_sec_min					real
	, seq_scans_sec_avg					real
	, seq_scans_sec_max					real
	, seq_tuples_read_sec_min			real
	, seq_tuples_read_sec_avg			real
	, seq_tuples_read_sec_max			real
	, index_scans_sec_min				real
	, index_scans_sec_avg				real
	, index_scans_sec_max				real
	, index_tuples_fetch_sec_min		real
	, index_tuples_fetch_sec_avg		real
	, index_tuples_fetch_sec_max		real
	, tuples_inserted_sec_min			real
	, tuples_inserted_sec_avg			real
	, tuples_inserted_sec_max			real
	, tuples_updated_sec_min			real
	, tuples_updated_sec_avg			real
	, tuples_updated_sec_max			real
	, tuples_deleted_sec_min			real
	, tuples_deleted_sec_avg			real
	, tuples_deleted_sec_max			real
	, heap_blocks_read_sec_min			real
	, heap_blocks_read_sec_avg			real
	, heap_blocks_read_sec_max			real
	, heap_blocks_hit_sec_min			real
	, heap_blocks_hit_sec_avg			real
	, heap_blocks_hit_sec_max			real
	, index_blocks_read_sec_min			real
	, index_blocks_read_sec_avg			real
	, index_blocks_read_sec_max			real
	, index_blocks_hit_sec_min			real
	, index_blocks_hit_sec_avg			real
	, index_blocks_hit_sec_max			real
	, toast_blocks_read_sec_min			real
	, toast_blocks_read_sec_avg			real
	, toast_blocks_read_sec_max			real
	, toast_blocks_hit_sec_min			real
	, toast_blocks_hit_sec_avg			real
	, toast_blocks_hit_sec_max			real
	, toast_index_blocks_read_sec_min	real
	, toast_index_blocks_read_sec_avg	real
	, toast_index_blocks_read_sec_max	real
	, toast_index_blocks_hit_sec_min	real
	, toast_index_blocks_hit_sec_avg	real
	, toast_index_blocks_hit_sec_max	real
	, last_vacuum_age_min				interval
	, last_vacuum_age_avg				interval
	, last_vacuum_age_max				interval
	, last_autovacuum_age_min			interval
	, last_autovacuum_age_avg			interval
	, last_autovacuum_age_max			interval
	, last_analyze_age_min				interval
	, last_analyze_age_avg				interval
	, last_analyze_age_max				interval
	, last_autoanalyze_age_min			interval
	, last_autoanalyze_age_avg			interval
	, last_autoanalyze_age_max			interval
	, PRIMARY KEY (bucket_id, table_id)
);
INSERT INTO rrs.source( source_name, insert_table, source_table, source_timestamptz_field, group_clause
		, insert_aggregate_fields
		, primary_aggregate
		, rrs_aggregate
	) VALUES( 'Tables', 'rrs_data.table', 'delta.table_sec', 'captured_at', 'table_id'
		, $$
		seq_scans_sec_min
		, seq_scans_sec_avg
		, seq_scans_sec_max
		, seq_tuples_read_sec_min
		, seq_tuples_read_sec_avg
		, seq_tuples_read_sec_max
		, index_scans_sec_min
		, index_scans_sec_avg
		, index_scans_sec_max
		, index_tuples_fetch_sec_min
		, index_tuples_fetch_sec_avg
		, index_tuples_fetch_sec_max
		, tuples_inserted_sec_min
		, tuples_inserted_sec_avg
		, tuples_inserted_sec_max
		, tuples_updated_sec_min
		, tuples_updated_sec_avg
		, tuples_updated_sec_max
		, tuples_deleted_sec_min
		, tuples_deleted_sec_avg
		, tuples_deleted_sec_max
		, heap_blocks_read_sec_min
		, heap_blocks_read_sec_avg
		, heap_blocks_read_sec_max
		, heap_blocks_hit_sec_min
		, heap_blocks_hit_sec_avg
		, heap_blocks_hit_sec_max
		, index_blocks_read_sec_min
		, index_blocks_read_sec_avg
		, index_blocks_read_sec_max
		, index_blocks_hit_sec_min
		, index_blocks_hit_sec_avg
		, index_blocks_hit_sec_max
		, toast_blocks_read_sec_min
		, toast_blocks_read_sec_avg
		, toast_blocks_read_sec_max
		, toast_blocks_hit_sec_min
		, toast_blocks_hit_sec_avg
		, toast_blocks_hit_sec_max
		, toast_index_blocks_read_sec_min
		, toast_index_blocks_read_sec_avg
		, toast_index_blocks_read_sec_max
		, toast_index_blocks_hit_sec_min
		, toast_index_blocks_hit_sec_avg
		, toast_index_blocks_hit_sec_max
		, last_vacuum_age_min
		, last_vacuum_age_avg
		, last_vacuum_age_max
		, last_autovacuum_age_min
		, last_autovacuum_age_avg
		, last_autovacuum_age_max
		, last_analyze_age_min
		, last_analyze_age_avg
		, last_analyze_age_max
		, last_autoanalyze_age_min
		, last_autoanalyze_age_avg
		, last_autoanalyze_age_max
		$$
		-- primary_aggregate
		, $$
		min(seq_scans_sec)
		, avg(seq_scans_sec)
		, max(seq_scans_sec)
		, min(seq_tuples_read_sec)
		, avg(seq_tuples_read_sec)
		, max(seq_tuples_read_sec)
		, min(index_scans_sec)
		, avg(index_scans_sec)
		, max(index_scans_sec)
		, min(index_tuples_fetch_sec)
		, avg(index_tuples_fetch_sec)
		, max(index_tuples_fetch_sec)
		, min(tuples_inserted_sec)
		, avg(tuples_inserted_sec)
		, max(tuples_inserted_sec)
		, min(tuples_updated_sec)
		, avg(tuples_updated_sec)
		, max(tuples_updated_sec)
		, min(tuples_deleted_sec)
		, avg(tuples_deleted_sec)
		, max(tuples_deleted_sec)
		, min(heap_blocks_read_sec)
		, avg(heap_blocks_read_sec)
		, max(heap_blocks_read_sec)
		, min(heap_blocks_hit_sec)
		, avg(heap_blocks_hit_sec)
		, max(heap_blocks_hit_sec)
		, min(index_blocks_read_sec)
		, avg(index_blocks_read_sec)
		, max(index_blocks_read_sec)
		, min(index_blocks_hit_sec)
		, avg(index_blocks_hit_sec)
		, max(index_blocks_hit_sec)
		, min(toast_blocks_read_sec)
		, avg(toast_blocks_read_sec)
		, max(toast_blocks_read_sec)
		, min(toast_blocks_hit_sec)
		, avg(toast_blocks_hit_sec)
		, max(toast_blocks_hit_sec)
		, min(toast_index_blocks_read_sec)
		, avg(toast_index_blocks_read_sec)
		, max(toast_index_blocks_read_sec)
		, min(toast_index_blocks_hit_sec)
		, avg(toast_index_blocks_hit_sec)
		, max(toast_index_blocks_hit_sec)
		, min(last_vacuum_age)
		, avg(last_vacuum_age)
		, max(last_vacuum_age)
		, min(last_autovacuum_age)
		, avg(last_autovacuum_age)
		, max(last_autovacuum_age)
		, min(last_analyze_age)
		, avg(last_analyze_age)
		, max(last_analyze_age)
		, min(last_autoanalyze_age)
		, avg(last_autoanalyze_age)
		, max(last_autoanalyze_age)
		$$
		-- primary_aggregate
		, $$
		min(seq_scans_sec_min)
		, avg(seq_scans_sec_avg)
		, max(seq_scans_sec_max)
		, min(seq_tuples_read_sec_min)
		, avg(seq_tuples_read_sec_avg)
		, max(seq_tuples_read_sec_max)
		, min(index_scans_sec_min)
		, avg(index_scans_sec_avg)
		, max(index_scans_sec_max)
		, min(index_tuples_fetch_sec_min)
		, avg(index_tuples_fetch_sec_avg)
		, max(index_tuples_fetch_sec_max)
		, min(tuples_inserted_sec_min)
		, avg(tuples_inserted_sec_avg)
		, max(tuples_inserted_sec_max)
		, min(tuples_updated_sec_min)
		, avg(tuples_updated_sec_avg)
		, max(tuples_updated_sec_max)
		, min(tuples_deleted_sec_min)
		, avg(tuples_deleted_sec_avg)
		, max(tuples_deleted_sec_max)
		, min(heap_blocks_read_sec_min)
		, avg(heap_blocks_read_sec_avg)
		, max(heap_blocks_read_sec_max)
		, min(heap_blocks_hit_sec_min)
		, avg(heap_blocks_hit_sec_avg)
		, max(heap_blocks_hit_sec_max)
		, min(index_blocks_read_sec_min)
		, avg(index_blocks_read_sec_avg)
		, max(index_blocks_read_sec_max)
		, min(index_blocks_hit_sec_min)
		, avg(index_blocks_hit_sec_avg)
		, max(index_blocks_hit_sec_max)
		, min(toast_blocks_read_sec_min)
		, avg(toast_blocks_read_sec_avg)
		, max(toast_blocks_read_sec_max)
		, min(toast_blocks_hit_sec_min)
		, avg(toast_blocks_hit_sec_avg)
		, max(toast_blocks_hit_sec_max)
		, min(toast_index_blocks_read_sec_min)
		, avg(toast_index_blocks_read_sec_avg)
		, max(toast_index_blocks_read_sec_max)
		, min(toast_index_blocks_hit_sec_min)
		, avg(toast_index_blocks_hit_sec_avg)
		, max(toast_index_blocks_hit_sec_max)
		, min(last_vacuum_age_min)
		, avg(last_vacuum_age_avg)
		, max(last_vacuum_age_max)
		, min(last_autovacuum_age_min)
		, avg(last_autovacuum_age_avg)
		, max(last_autovacuum_age_max)
		, min(last_analyze_age_min)
		, avg(last_analyze_age_avg)
		, max(last_analyze_age_max)
		, min(last_autoanalyze_age_min)
		, avg(last_autoanalyze_age_avg)
		, max(last_autoanalyze_age_max)
		$$
	)
;

CREATE TABLE rrs_data.index(
	bucket_id					int		NOT NULL REFERENCES rrs.bucket(bucket_id) ON DELETE CASCADE
	, index_id					int		NOT NULL REFERENCES raw.indexes(index_id) ON DELETE CASCADE
	, scans_sec_min				real
	, scans_sec_avg				real
	, scans_sec_max				real
	, tuples_read_sec_min		real
	, tuples_read_sec_avg		real
	, tuples_read_sec_max		real
	, tuples_fetched_sec_min	real
	, tuples_fetched_sec_avg	real
	, tuples_fetched_sec_max	real
	, blocks_read_sec_min		real
	, blocks_read_sec_avg		real
	, blocks_read_sec_max		real
	, blocks_hit_sec_min		real
	, blocks_hit_sec_avg		real
	, blocks_hit_sec_max		real
	, PRIMARY KEY (bucket_id, index_id)
);
INSERT INTO rrs.source( source_name, insert_table, source_table, source_timestamptz_field, group_clause
		, insert_aggregate_fields
		, primary_aggregate
		, rrs_aggregate
	) VALUES( 'Indexes', 'rrs_data.index', 'delta.index_sec', 'captured_at', 'index_id'
		, $$
		scans_sec_min
		, scans_sec_avg
		, scans_sec_max
		, tuples_read_sec_min
		, tuples_read_sec_avg
		, tuples_read_sec_max
		, tuples_fetched_sec_min
		, tuples_fetched_sec_avg
		, tuples_fetched_sec_max
		, blocks_read_sec_min
		, blocks_read_sec_avg
		, blocks_read_sec_max
		, blocks_hit_sec_min
		, blocks_hit_sec_avg
		, blocks_hit_sec_max
		$$
		-- primary_aggregate
		, $$
		min(scans_sec)
		, avg(scans_sec)
		, max(scans_sec)
		, min(tuples_read_sec)
		, avg(tuples_read_sec)
		, max(tuples_read_sec)
		, min(tuples_fetched_sec)
		, avg(tuples_fetched_sec)
		, max(tuples_fetched_sec)
		, min(blocks_read_sec)
		, avg(blocks_read_sec)
		, max(blocks_read_sec)
		, min(blocks_hit_sec)
		, avg(blocks_hit_sec)
		, max(blocks_hit_sec)
		$$
		-- primary_aggregate
		, $$
		min(scans_sec_min)
		, avg(scans_sec_avg)
		, max(scans_sec_max)
		, min(tuples_read_sec_min)
		, avg(tuples_read_sec_avg)
		, max(tuples_read_sec_max)
		, min(tuples_fetched_sec_min)
		, avg(tuples_fetched_sec_avg)
		, max(tuples_fetched_sec_max)
		, min(blocks_read_sec_min)
		, avg(blocks_read_sec_avg)
		, max(blocks_read_sec_max)
		, min(blocks_hit_sec_min)
		, avg(blocks_hit_sec_avg)
		, max(blocks_hit_sec_max)
		$$
	)
;

CREATE TABLE rrs_data.sequence(
	bucket_id					int		NOT NULL REFERENCES rrs.bucket(bucket_id) ON DELETE CASCADE
	, sequence_id				int		NOT NULL REFERENCES raw.sequences(sequence_id) ON DELETE CASCADE
	, blocks_read_sec_min		real
	, blocks_read_sec_avg		real
	, blocks_read_sec_max		real
	, blocks_hit_sec_min		real
	, blocks_hit_sec_avg		real
	, blocks_hit_sec_max		real
	, PRIMARY KEY (bucket_id, sequence_id)
);
INSERT INTO rrs.source( source_name, insert_table, source_table, source_timestamptz_field, group_clause
		, insert_aggregate_fields
		, primary_aggregate
		, rrs_aggregate
	) VALUES( 'Sequences', 'rrs_data.sequence', 'delta.sequence_sec', 'captured_at', 'sequence_id'
		, $$
		blocks_read_sec_min
		, blocks_read_sec_avg
		, blocks_read_sec_max
		, blocks_hit_sec_min
		, blocks_hit_sec_avg
		, blocks_hit_sec_max
		$$
		-- primary_aggregate
		, $$
		min(blocks_read_sec)
		, avg(blocks_read_sec)
		, max(blocks_read_sec)
		, min(blocks_hit_sec)
		, avg(blocks_hit_sec)
		, max(blocks_hit_sec)
		$$
		-- primary_aggregate
		, $$
		min(blocks_read_sec_min)
		, avg(blocks_read_sec_avg)
		, max(blocks_read_sec_max)
		, min(blocks_hit_sec_min)
		, avg(blocks_hit_sec_avg)
		, max(blocks_hit_sec_max)
		$$
	)
;


-- vi: noexpandtab sw=4 ts=4
