/*
 * Copyright (c) 2007, Jim C. Nasby (decibel@decibel.org)
 * All rights reserved.
 *
 * $Id: rrs.sql 34 2007-11-16 19:02:59Z decibel $
 */

/*
 * Useful vi commands:
 *
 * Takes all fields ending in '_min' and creates avg and max versions
 * s/\(\s*, \S*\)_min\(.*\)/&^M\1_avg\2^M\1_max\2/
 *
 * Run (min|avg|max) function on fields selected, based on tailing _ value, striping tail.
 * IE: idle_min becomes min(idle)
 * s/\v(\w*)_(min|avg|max)/\2(\1)/
 *
 * Same as above, but doesn't strip tail
 * s/\v(\w*)_(min|avg|max)/\2(\1_\2)/
 */

CREATE OR REPLACE VIEW public.database_stats AS
	SELECT b.rrs_id, b.end_time, b.previous_end_time, d.*
		FROM rrs.bucket_trimmed b
			LEFT JOIN rrs_data.database d USING( bucket_id )
;
CREATE OR REPLACE VIEW public.database_stats_full AS
	SELECT d.cluster_name, d.cluster_id, d.database_oid, d.database_name, r.*
		FROM public.databases d NATURAL JOIN public.database_stats r
;

CREATE OR REPLACE VIEW public.table_stats AS
	SELECT b.rrs_id, b.end_time, b.previous_end_time, d.*
		FROM rrs.bucket_trimmed b
			LEFT JOIN rrs_data.table d USING( bucket_id )
;
CREATE OR REPLACE VIEW public.table_stats_full AS
	SELECT t.cluster_name, t.cluster_id, t.database_oid, t.database_name
			, t.database_id, t.table_oid, t.schema_name, t.table_name, r.*
		FROM public.tables t NATURAL JOIN public.table_stats r
;

CREATE OR REPLACE VIEW public.index_stats AS
	SELECT b.rrs_id, b.end_time, b.previous_end_time, d.*
		FROM rrs.bucket_trimmed b
			LEFT JOIN rrs_data.index d USING( bucket_id )
;
-- Note there's no schema_name here
CREATE OR REPLACE VIEW public.index_stats_full AS
	SELECT t.cluster_name, t.cluster_id, t.database_oid, t.database_name
			, t.database_id, t.index_oid, t.index_name, r.*
		FROM public.indexes t NATURAL JOIN public.index_stats r
;

CREATE OR REPLACE VIEW public.sequence_stats AS
	SELECT b.rrs_id, b.end_time, b.previous_end_time, d.*
		FROM rrs.bucket_trimmed b
			LEFT JOIN rrs_data.sequence d USING( bucket_id )
;
CREATE OR REPLACE VIEW public.sequence_stats_full AS
	SELECT t.cluster_name, t.cluster_id, t.database_oid, t.database_name
			, t.database_id, t.sequence_oid, t.schema_name, t.sequence_name, r.*
		FROM public.sequences t NATURAL JOIN public.sequence_stats r
;


-- vi: noexpandtab sw=4 ts=4
