/*
 * Copyright (c) 2007, Jim C. Nasby (decibel@decibel.org)
 * All rights reserved.
 *
 * $Id: views.sql 35 2007-12-29 16:16:14Z decibel $
 */

CREATE OR REPLACE VIEW public.databases AS
	SELECT cluster_name, d.* FROM public.clusters c NATURAL JOIN raw.databases d
;
GRANT SELECT ON public.databases TO public;

CREATE OR REPLACE VIEW public.tables AS
	SELECT cluster_name, cluster_id, database_oid, database_name, t.* FROM public.databases NATURAL JOIN raw.tables t
;
GRANT SELECT ON public.tables TO public;

CREATE OR REPLACE VIEW public.indexes AS
	SELECT cluster_name, cluster_id, database_oid, database_name, i.* FROM public.databases NATURAL JOIN raw.indexes i
;
GRANT SELECT ON public.indexes TO public;

CREATE OR REPLACE VIEW public.sequences AS
	SELECT cluster_name, cluster_id, database_oid, database_name, s.* FROM public.databases NATURAL JOIN raw.sequences s
;
GRANT SELECT ON public.sequences TO public;

-- Do left joins in case anything is using this view only to get captured time info
CREATE OR REPLACE VIEW raw.database_stats_capture AS
	SELECT c.cluster_id, c.captured_at, c.previous_capture_id, s.*
		FROM raw.database_capture c NATURAL LEFT JOIN raw.database_stats s
;
CREATE OR REPLACE VIEW raw.table_stats_capture AS
	SELECT c.database_id, c.captured_at, c.previous_capture_id, s.*
		FROM raw.table_capture c NATURAL LEFT JOIN raw.table_stats s
;
CREATE OR REPLACE VIEW raw.index_stats_capture AS
	SELECT c.database_id, c.captured_at, c.previous_capture_id, s.*
		FROM raw.index_capture c NATURAL LEFT JOIN raw.index_stats s
;
CREATE OR REPLACE VIEW raw.sequence_stats_capture AS
	SELECT c.database_id, c.captured_at, c.previous_capture_id, s.*
		FROM raw.sequence_capture c NATURAL LEFT JOIN raw.sequence_stats s
;

CREATE OR REPLACE VIEW delta.database_stats AS
	SELECT c.database_id, c.captured_at, p.captured_at AS previous_time
			, c.num_backends, c.idle, c.idle_in_transaction
			, delta.delta( p.commits, c.commits) AS commits
			, delta.delta( p.rollbacks, c.rollbacks) AS rollbacks
			, delta.delta( p.blocks_read, c.blocks_read) AS blocks_read
			, delta.delta( p.blocks_hit, c.blocks_hit) AS blocks_hit
		FROM raw.database_stats_capture c
			LEFT JOIN raw.database_stats_capture p ON ( c.database_id = p.database_id AND c.previous_capture_id = p.capture_id )
		-- This where condition greatly speeds up queries that are only pulling a small portion of the data in the table
		WHERE p.captured_at < c.captured_at
;
CREATE OR REPLACE VIEW delta.table_stats AS
	SELECT c.table_id, c.captured_at, p.captured_at AS previous_time
			, delta.delta( p.seq_scans, c.seq_scans ) AS seq_scans
			, delta.delta( p.seq_tuples_read, c.seq_tuples_read ) AS seq_tuples_read
			, delta.delta( p.index_scans, c.index_scans ) AS index_scans
			, delta.delta( p.index_tuples_fetch, c.index_tuples_fetch ) AS index_tuples_fetch
			, delta.delta( p.tuples_inserted, c.tuples_inserted ) AS tuples_inserted
			, delta.delta( p.tuples_updated, c.tuples_updated ) AS tuples_updated
			, delta.delta( p.tuples_deleted, c.tuples_deleted ) AS tuples_deleted
			, delta.delta( p.heap_blocks_read, c.heap_blocks_read ) AS heap_blocks_read
			, delta.delta( p.heap_blocks_hit, c.heap_blocks_hit ) AS heap_blocks_hit
			, delta.delta( p.index_blocks_read, c.index_blocks_read ) AS index_blocks_read
			, delta.delta( p.index_blocks_hit, c.index_blocks_hit ) AS index_blocks_hit
			, delta.delta( p.toast_blocks_read, c.toast_blocks_read ) AS toast_blocks_read
			, delta.delta( p.toast_blocks_hit, c.toast_blocks_hit ) AS toast_blocks_hit
			, delta.delta( p.toast_index_blocks_read, c.toast_index_blocks_read ) AS toast_index_blocks_read
			, delta.delta( p.toast_index_blocks_hit, c.toast_index_blocks_hit ) AS toast_index_blocks_hit
			, c.last_vacuum, c.last_autovacuum, c.last_analyze, c.last_autoanalyze
		FROM raw.table_stats_capture c
			LEFT JOIN raw.table_stats_capture p ON ( c.table_id = p.table_id AND c.previous_capture_id = p.capture_id )
		WHERE p.captured_at < c.captured_at
;
CREATE OR REPLACE VIEW delta.index_stats AS
	SELECT c.index_id, c.captured_at, p.captured_at AS previous_time
			, delta.delta( p.scans, c.scans ) AS scans
			, delta.delta( p.tuples_read, c.tuples_read ) AS tuples_read
			, delta.delta( p.tuples_fetched, c.tuples_fetched ) AS tuples_fetched
			, delta.delta( p.blocks_read, c.blocks_read ) AS blocks_read
			, delta.delta( p.blocks_hit, c.blocks_hit ) AS blocks_hit
		FROM raw.index_stats_capture c
			LEFT JOIN raw.index_stats_capture p ON ( c.index_id = p.index_id AND c.previous_capture_id = p.capture_id )
		WHERE p.captured_at < c.captured_at
;
CREATE OR REPLACE VIEW delta.sequence_stats AS
	SELECT c.sequence_id, c.captured_at, p.captured_at AS previous_time
			, delta.delta( p.blocks_read, c.blocks_read ) AS blocks_read
			, delta.delta( p.blocks_hit, c.blocks_hit ) AS blocks_hit
		FROM raw.sequence_stats_capture c
			LEFT JOIN raw.sequence_stats_capture p ON ( c.sequence_id = p.sequence_id AND c.previous_capture_id = p.capture_id )
		WHERE p.captured_at < c.captured_at
;

SET search_path = delta;

CREATE OR REPLACE VIEW database_sec AS
	SELECT database_id, captured_at, previous_time
			, num_backends, idle, idle_in_transaction
			, per_second( previous_time, captured_at, commits ) AS commits_sec
			, per_second( previous_time, captured_at, rollbacks ) AS rollbacks_sec
			, per_second( previous_time, captured_at, blocks_read ) AS blocks_read_sec
			, per_second( previous_time, captured_at, blocks_hit ) AS blocks_hit_sec
		FROM database_stats
;

CREATE OR REPLACE VIEW table_sec AS
	SELECT table_id, captured_at, previous_time
			, per_second( previous_time, captured_at, seq_scans ) AS seq_scans_sec
			, per_second( previous_time, captured_at, seq_tuples_read ) AS seq_tuples_read_sec
			, per_second( previous_time, captured_at, index_scans ) AS index_scans_sec
			, per_second( previous_time, captured_at, index_tuples_fetch ) AS index_tuples_fetch_sec
			, per_second( previous_time, captured_at, tuples_inserted ) AS tuples_inserted_sec
			, per_second( previous_time, captured_at, tuples_updated ) AS tuples_updated_sec
			, per_second( previous_time, captured_at, tuples_deleted ) AS tuples_deleted_sec
			, per_second( previous_time, captured_at, heap_blocks_read ) AS heap_blocks_read_sec
			, per_second( previous_time, captured_at, heap_blocks_hit ) AS heap_blocks_hit_sec
			, per_second( previous_time, captured_at, index_blocks_read ) AS index_blocks_read_sec
			, per_second( previous_time, captured_at, index_blocks_hit ) AS index_blocks_hit_sec
			, per_second( previous_time, captured_at, toast_blocks_read ) AS toast_blocks_read_sec
			, per_second( previous_time, captured_at, toast_blocks_hit ) AS toast_blocks_hit_sec
			, per_second( previous_time, captured_at, toast_index_blocks_read ) AS toast_index_blocks_read_sec
			, per_second( previous_time, captured_at, toast_index_blocks_hit ) AS toast_index_blocks_hit_sec
			, age( captured_at, last_vacuum ) AS last_vacuum_age
			, age( captured_at, last_autovacuum ) AS last_autovacuum_age
			, age( captured_at, last_analyze ) AS last_analyze_age
			, age( captured_at, last_autoanalyze ) AS last_autoanalyze_age
		FROM table_stats
;

CREATE OR REPLACE VIEW index_sec AS
	SELECT index_id, captured_at, previous_time
			, per_second( previous_time, captured_at, scans ) AS scans_sec
			, per_second( previous_time, captured_at, tuples_read ) AS tuples_read_sec
			, per_second( previous_time, captured_at, tuples_fetched ) AS tuples_fetched_sec
			, per_second( previous_time, captured_at, blocks_read ) AS blocks_read_sec
			, per_second( previous_time, captured_at, blocks_hit ) AS blocks_hit_sec
		FROM index_stats
;

CREATE OR REPLACE VIEW sequence_sec AS
	SELECT sequence_id, captured_at, previous_time
			, per_second( previous_time, captured_at, blocks_read ) AS blocks_read_sec
			, per_second( previous_time, captured_at, blocks_hit ) AS blocks_hit_sec
		FROM sequence_stats
;


-- vi: noexpandtab sw=4 ts=4
